//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
#ifndef FEATURE_FORCE_TORQUE_PREINT_H_
#define FEATURE_FORCE_TORQUE_PREINT_H_

//Wolf includes
#include <core/feature/feature_base.h>
#include <core/common/wolf.h>

//std includes

namespace wolf {

//WOLF_PTR_TYPEDEFS(CaptureImu);
WOLF_PTR_TYPEDEFS(FeatureForceTorquePreint);

class FeatureForceTorquePreint : public FeatureBase
{
    public:

        /** \brief Constructor from and measures
         *
         * \param _measurement the measurement
         * \param _meas_covariance the noise of the measurement
         * \param _J_delta_bias Jacobians of preintegrated delta wrt [pbc,gyro] biases
         * \param _pbc_bias COM position relative to bias bias of origin frame time
         * \param _gyro_bias gyroscope bias of origin frame time
         * \param _cap_ftpreint_ptr pointer to parent CaptureMotion (CaptureForceTorque)
         */
        FeatureForceTorquePreint(const Eigen::VectorXd& _delta_preintegrated,
                                 const Eigen::MatrixXd& _delta_preintegrated_covariance,
                                 const Eigen::Vector6d& _biases_preint,
                                 const Eigen::Matrix<double,12,6>& _J_delta_biases);

//        /** \brief Constructor from capture pointer
//         *
//         * \param _cap_imu_ptr pointer to parent CaptureMotion
//         */
//        FeatureForceTorquePreint(CaptureMotionPtr _cap_imu_ptr);

        ~FeatureForceTorquePreint() override = default;

        const Eigen::Vector3d&               getPbcBiasPreint()  const;
        const Eigen::Vector3d&               getGyroBiasPreint() const;
        const Eigen::Matrix<double, 12, 6>&  getJacobianBias()   const;

    private:

        // Used biases
        Eigen::Vector3d pbc_bias_preint_;       ///< Acceleration bias used for delta preintegration
        Eigen::Vector3d gyro_bias_preint_;      ///< Gyrometer bias used for delta preintegration

        Eigen::Matrix<double, 12, 6> J_delta_bias_;

    public:
      EIGEN_MAKE_ALIGNED_OPERATOR_NEW;
};

inline const Eigen::Vector3d& FeatureForceTorquePreint::getPbcBiasPreint() const
{
    return pbc_bias_preint_;
}

inline const Eigen::Vector3d& FeatureForceTorquePreint::getGyroBiasPreint() const
{
    return gyro_bias_preint_;
}

inline const Eigen::Matrix<double, 12, 6>& FeatureForceTorquePreint::getJacobianBias() const
{
    return J_delta_bias_;
}

} // namespace wolf

#endif // FEATURE_FORCE_TORQUE_PREINT_H_
